@@ -49,6 +49,7 @@ pub struct Session {
     sender: UnboundedSender<Reply>,
     channels: HashMap<ChannelId, UnboundedSender<OpenChannelMsg>>,
     target_window_size: u32,
+    pending_reads: Vec<CryptoVec>,
 }
 
 impl Drop for Session {
@@ -807,6 +808,7 @@ where
         receiver,
         sender: sender2,
         channels: HashMap::new(),
+        pending_reads: Vec::new(),
     };
     session.read_ssh_id(sshid)?;
     let (encrypted_signal, encrypted_recv) = tokio::sync::oneshot::channel();
