@@ -305,6 +305,9 @@ macro_rules! push_packet {
         x
     }};
 }
+
+type Sha256Hash = generic_array::GenericArray<u8, <sha2::Sha256 as digest::FixedOutputDirty>::OutputSize>;
+
 mod session;
 
 /// Server side of this library.
@@ -422,6 +425,7 @@ pub enum Error {
     Join(#[from] tokio::task::JoinError),
 
     #[error(transparent)]
+    #[cfg(feature = "openssl")]
     Openssl(#[from] openssl::error::ErrorStack),
 
     #[error(transparent)]
