@@ -193,6 +193,18 @@ impl Encrypted {
         pending_size
     }
 
+    pub fn flush_all_pending(&mut self) {
+        for (_, channel) in self.channels.iter_mut() {
+            while let Some((buf, a, from)) = channel.pending_data.pop_front() {
+                let size = Self::data_noqueue(&mut self.write, channel, &buf, from);
+                if from + size < buf.len() {
+                    channel.pending_data.push_front((buf, a, from + size));
+                    break;
+                }
+            }
+        }
+    }
+
     pub fn has_pending_data(&self, channel: ChannelId) -> bool {
         if let Some(channel) = self.channels.get(&channel) {
             !channel.pending_data.is_empty()
@@ -240,7 +252,7 @@ impl Encrypted {
     pub fn data(&mut self, channel: ChannelId, buf0: CryptoVec) {
         if let Some(channel) = self.channels.get_mut(&channel) {
             assert!(channel.confirmed);
-            if !channel.pending_data.is_empty() {
+            if !channel.pending_data.is_empty() || self.rekey.is_some() {
                 channel.pending_data.push_back((buf0, None, 0));
                 return;
             }
